﻿using System;
using System.Configuration;
using System.Data;
using System.Data.SqlClient;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

public partial class _Default : System.Web.UI.Page
{
    // Global variable to hold the State
    private String strCurrentState = "";
    protected void Page_Load(object sender, EventArgs e)
    {
        if (Page.IsPostBack == false)
            PopulateListView();
    }

    protected void NewAuthor(Object sender, EventArgs e)
    {
        ListView1.EditIndex = -1;
        ListView1.InsertItemPosition = InsertItemPosition.LastItem;
        ((LinkButton)sender).Visible = false;
        PopulateListView();
    }

    protected void CommandList(Object sender, ListViewCommandEventArgs e)
    {
        if (e.CommandName.ToUpper() == "SAVE")
        {
            SaveList(e.Item);
        }
    }

    protected void InsertList(Object sender, ListViewInsertEventArgs e)
    {
        ListView1.InsertItemPosition = InsertItemPosition.None; 
    }

    protected void CancelInsertMode()
    {
        ListView1.InsertItemPosition = InsertItemPosition.None;
        ((LinkButton)ListView1.FindControl("lnkNew")).Visible = true;
        PopulateListView();
    }

    protected void DataBoundList(Object sender, ListViewItemEventArgs e)
    {
        if (e.Item.ItemType == ListViewItemType.DataItem)
        {
            LinkButton lnkDelete = (LinkButton)e.Item.FindControl("lnkDelete");
            if (lnkDelete != null)
            {
                lnkDelete.Attributes.Add("onclick", "return confirm('Are you sure you want to delete this author?');");
            }

            // Get a handle to the ddlState DropDownList control
            DropDownList ddl = (DropDownList)e.Item.FindControl("ddlState");
            // Make sure we have the handle !
            if (ddl != null)
            {
                ddl.SelectedIndex = ddl.Items.IndexOf(ddl.Items.FindByText(strCurrentState));
            }
        }
   }

    // Invoked when the Edit Link is clicked
    protected void EditList(Object sender, ListViewEditEventArgs e)
    {
        // Set the Listview to Editmode
        ListView1.EditIndex = e.NewEditIndex;

        // Get the Value for State
        Label lblTemp = (Label)ListView1.Items[e.NewEditIndex].FindControl("lblState");
        strCurrentState = lblTemp.Text;

        // Bind the view 
        PopulateListView();
    }

    // Invoked when the Delete Link is clicked
    protected void DeleteList(Object sender, ListViewDeleteEventArgs e)
    {
        // Get the current item being deleted
        ListViewItem myItem = ListView1.Items[e.ItemIndex];

        // get the author id which is being deleted
        Label lblAuthorID = (Label)myItem.FindControl("lblItemauid");

        DeleteAuthors(lblAuthorID.Text);
        PopulateListView();
    }

    // Invoked when the Cancel Link is clicked
    protected void CancelList(Object sender, ListViewCancelEventArgs e)
    {
        if (e.CancelMode == ListViewCancelMode.CancelingEdit)
        {
            ListView1.EditIndex = -1;
            PopulateListView();
        }
        else if (e.CancelMode == ListViewCancelMode.CancelingInsert)
        {
            CancelInsertMode();
        }
    }

    // Invoked when the Update Link is clicked
    protected void UpdateList(Object sender, ListViewUpdateEventArgs e)
    {
        // Get the current item being edited
        ListViewItem myItem = ListView1.Items[ListView1.EditIndex];

        // get the author id which is being edited
        Label lblAuthorID = (Label)myItem.FindControl("lblauid");

        // Get the new values
        TextBox txtLname = (TextBox)myItem.FindControl("txtLName");
        TextBox txtFname = (TextBox)myItem.FindControl("txtFName");
        DropDownList ddlState = (DropDownList)myItem.FindControl("ddlState");
        TextBox txtCity = (TextBox)myItem.FindControl("txtCity");

        // invoke the method which actually updates the value in the Database
        UpdateAuthors(0, lblAuthorID.Text, txtLname.Text, txtFname.Text, ddlState.SelectedValue.ToString(), txtCity.Text);

        // get out of the edit mode
        ListView1.EditIndex = -1;

        // bind the listview
        PopulateListView();
    }

    // Invoked when the Save Link is clicked
    protected void SaveList(ListViewItem e)
    {
        // get a new author ID
        String strAuthorID = Guid.NewGuid().ToString();

        // Get the new values
        TextBox txtLname = (TextBox)e.FindControl("txtLName");
        TextBox txtFname = (TextBox)e.FindControl("txtFName");
        DropDownList ddlState = (DropDownList)e.FindControl("ddlState");
        TextBox txtCity = (TextBox)e.FindControl("txtCity");

        // invoke the UpdateAuthors method, which actually updates the value in the Database
        UpdateAuthors (1, strAuthorID, txtLname.Text, txtFname.Text, ddlState.SelectedValue.ToString(), txtCity.Text);

        // Hide the text boxes
        CancelInsertMode();

        // bind the listview
        PopulateListView();
    }

    protected void PopulateListView()
    {
        SqlConnection myConn = CreateConnection();
        // CreateConnetion is a custom method which returns SQL connection.
        SqlCommand myCmd = new SqlCommand("pubs_get_authors", myConn);
        myCmd.CommandType = CommandType.StoredProcedure;
        try
        {
            myConn.Open();
            ListView1.DataSource = myCmd.ExecuteReader (CommandBehavior.CloseConnection);
        }
        catch (SqlException se)
        {
            lblMsg.Text = se.Message;
        }
        catch (Exception e)
        {
            lblMsg.Text = e.Message;
        }
        finally
        { }
        ListView1.DataBind();
        myConn.Close();
    }

    protected void UpdateAuthors(int intFlag, String strauid, String strlname, String strfname, String strstate, String strcity)
    {
        SqlConnection myConn = CreateConnection();
        SqlCommand myCmd = null;

        // if intFlag contains zero, then update the table
        if (intFlag == 0)
        {
            myCmd = new SqlCommand("pubs_update_authors", myConn);
        }
        // if intFlag contains one, then insert a new row to the table
        else if (intFlag == 1)
        {
            myCmd = new SqlCommand("pubs_insert_authors", myConn);
        }

        myCmd.CommandType = CommandType.StoredProcedure;

        SqlParameter myPrm1 = new SqlParameter("@auid", strauid);
        myCmd.Parameters.Add(myPrm1);

        SqlParameter myPrm2 = new SqlParameter("@lname", strlname);
        myCmd.Parameters.Add(myPrm2);

        SqlParameter myPrm3 = new SqlParameter("@fname", strfname);
        myCmd.Parameters.Add(myPrm3);

        SqlParameter myPrm4 = new SqlParameter("@state", strstate);
        myCmd.Parameters.Add(myPrm4);

        SqlParameter myPrm5 = new SqlParameter("@city", strcity);
        myCmd.Parameters.Add(myPrm5);

        try
        {
            myConn.Open();
            ListView1.DataSource = myCmd.ExecuteReader(CommandBehavior.CloseConnection);
        }
        catch (SqlException se)
        {
            lblMsg.Text = se.Message;
        }
        catch (Exception e)
        {
            lblMsg.Text = e.Message;
        }
        finally
        { }
        myConn.Close();
    }

    protected void DeleteAuthors(String strauid)
    {
        SqlConnection myConn = CreateConnection();
        SqlCommand myCmd = new SqlCommand("pubs_delete_authors", myConn);
        myCmd.CommandType = CommandType.StoredProcedure;

        SqlParameter myPrm1 = new SqlParameter("@auid", strauid);
        myCmd.Parameters.Add(myPrm1);

        try
        {
//          comment the following two lines to NOT Delete the authors physically from the table
          myConn.Open();
          ListView1.DataSource = myCmd.ExecuteReader(CommandBehavior.CloseConnection);
        }
        catch (SqlException se)
        {
            lblMsg.Text = se.Message;
        }
        catch (Exception e)
        {
            lblMsg.Text = e.Message;
        }
        finally
        { }
        myConn.Close();
    }

    protected SqlConnection CreateConnection()
    {
        ConnectionStringSettings settings = ConfigurationManager.ConnectionStrings["MyConnectionString"];
        String strConn = "";
        SqlConnection myConn = null;
        // If found, CONNECT to the DB.
        if (settings != null)
        {
            strConn = settings.ConnectionString;

            try
            {
                myConn = new SqlConnection(strConn);
            }
            catch (SqlException se)
            {
                lblMsg.Text = se.Message;
            }
            catch (Exception e)
            {
                lblMsg.Text = e.Message;
            }
            finally
            { }
            return myConn;
        }
        else
            return null;
    }
}